import yfinance as yf
import pandas as pd
import datetime as dt
import fundamentus
import requests_cache
import requests
import time, logging

# lib para coletar os dados de ativos da bolsa do site https://www.dadosdemercado.com.br/acoes
import requests
from bs4 import BeautifulSoup

# libs utilizadas na funcao para plotar o grafico de candlestick
import mplfinance as mpf
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from matplotlib.ticker import FormatStrFormatter, MaxNLocator

def obter_tickers(url):
    response = requests.get(url)
    soup = BeautifulSoup(response.content, 'html.parser')

    table = soup.find('table', {'class': 'normal-table', 'id': 'stocks'})
    tickers = []

    if table:
        rows = table.find_all('tr')
        for row in rows:
            link = row.find('a')
            if link and 'href' in link.attrs:
                ticker = link.text.strip()
                tickers.append(ticker)

    return tickers


def plot_candlestick(df, title, gatilho):
    # Converter a coluna 'date' para o índice do DataFrame e ordenar por data
    df.reset_index(inplace=True)
    df.set_index('date', inplace=True)
    df.sort_index(inplace=False)

    # Configurações de estilo e layout
    style = mpf.make_mpf_style(base_mpf_style='classic', rc={'axes.grid': True})

    # Criar o gráfico de candlestick com mplfinance
    fig, axes = mpf.plot(
        df, type='candle',
        style=style,
        title=f'Ativo - {title} - Gatilho: {gatilho}',
        ylabel='Preço (R$)', datetime_format='%d-%m-%Y',
        xrotation=45,
        ylabel_lower='Volume',
        figratio=(12, 6),
        volume=False,
        returnfig=True)  # Retorna a figura e os eixos para personalização

    # Ajustar a posição do eixo Y do preço para o lado esquerdo
    axes[0].yaxis.tick_left()  # Eixo Y do preço no lado esquerdo
    axes[0].yaxis.set_label_position("left")  # Rótulo do eixo Y no lado esquerdo
    axes[0].xaxis.set_major_locator(MaxNLocator(15))

    # Ajustar o layout para evitar sobreposição de rótulos
    fig.autofmt_xdate()

    # Exibir o gráfico
    plt.show()


def busca_detalhes_ativo(ticker):

  df = fundamentus.get_papel(f"{ticker}")

  print("####################################################################")
  print("#### Empresa: ",df.iloc[0]["Empresa"])
  print("#### Setor: ",df.iloc[0]["Setor"])
  print("#### Data ultima cotacao: ",df.iloc[0]["Data_ult_cot"])
  print("#### Cotacao: ",df.iloc[0]["Cotacao"])
  print("#### Margem Bruta: ",df.iloc[0]["Marg_Bruta"])
  print("#### Margem Liquida: ",df.iloc[0]["Marg_Liquida"])
  print("#### Margem EBITDA", df.iloc[0]["Marg_EBIT"])
  print("#### ROE: ",df.iloc[0]["ROE"])
  print("#### EBITDA ultimo 12 meses: ",df.iloc[0]["EBIT_12m"])
  print("#### Divida Bruta: ",df.iloc[0]["Div_Bruta"])
  print("#### Divida Liquida: ",df.iloc[0]["Div_Liquida"])
  print("####################################################################")
  print("                                                                    ")


def lista_acoes_gatilho_davy_landry(df_join):
  lista_acoes = []
  for ticker in df_join.ticker.unique():
    df_join_ticker = df_join[df_join["ticker"]==ticker].sort_values(by="date", ascending=False)

    # check daby landry
    check = df_join_ticker.iloc[0]["low"] < df_join_ticker.iloc[1]["low"] < df_join_ticker.iloc[2]["low"]
    if check:
      lista_acoes.append(ticker)

      busca_detalhes_ativo(ticker)

      df_join_ticker_filtered = df_join_ticker[df_join_ticker.ticker == ticker].sort_values(by="date", ascending=True)
      plot_candlestick(df_join_ticker_filtered, ticker, "Dave Landry")


  return lista_acoes

def lista_acoes_gatilho_123(df_join):
  lista_acoes = []
  for ticker in df_join.ticker.unique():
    df_join_ticker = df_join[df_join["ticker"]==ticker].sort_values(by="date", ascending=False)

    # check 123
    check = df_join_ticker.iloc[0]["low"] > df_join_ticker.iloc[1]["low"] < df_join_ticker.iloc[2]["low"]

    if check:
      lista_acoes.append(ticker)

      busca_detalhes_ativo(ticker)

      df_join_ticker_filtered = df_join_ticker[df_join_ticker.ticker == ticker].sort_values(by="date", ascending=True)
      plot_candlestick(df_join_ticker_filtered, ticker, "123")

  return lista_acoes

def lista_acoes_gatilho_inside_candle(df_join):
  lista_acoes = []
  for ticker in df_join.ticker.unique():
    df_join_ticker = df_join[df_join["ticker"]==ticker].sort_values(by="date", ascending=False)

    # check 123
    check1 = (df_join_ticker.iloc[0]["high"] < df_join_ticker.iloc[1]["open"]) and (df_join_ticker.iloc[0]["low"] > df_join_ticker.iloc[1]["close"])
    check2 = (df_join_ticker.iloc[0]["high"] < df_join_ticker.iloc[1]["close"]) and (df_join_ticker.iloc[0]["low"] > df_join_ticker.iloc[1]["open"])

    if check1 or check2:
      lista_acoes.append(ticker)

      busca_detalhes_ativo(ticker)

      df_join_ticker_filtered = df_join_ticker[df_join_ticker.ticker == ticker].sort_values(by="date", ascending=True)
      plot_candlestick(df_join_ticker_filtered, ticker, "Inside Candle")

  return lista_acoes


def coleta_dados_historicos(df, periodo):

  acoes_ibov_yfinance = [acoes+".SA" for acoes in df.index]

  df_list = []

  for ticker in acoes_ibov_yfinance:
    try:
      df_ohlcv = yf.download(ticker, period=periodo, progress=False)
      if df_ohlcv.shape[0] > 0:

        df_ohlcv.reset_index(inplace=True)
        df_ohlcv.columns = ["date", "close", "high", "low", "open", "volume"]
        df_ohlcv["ticker"] = ticker.split(".")[0]

        # remove os fins de semana
        df_ohlcv['date'] = pd.to_datetime(df_ohlcv['date'])
        df_ohlcv['day_of_week'] = df_ohlcv['date'].dt.dayofweek
        df_ohlcv = df_ohlcv[df_ohlcv['day_of_week'] < 5]
        df_ohlcv = df_ohlcv.drop(columns=['day_of_week'])

        df_list.append(df_ohlcv)
    except Exception as e:
      print(f"Ticker {ticker} not listed. Error: {e}")
      time.sleep(2)
      continue

  # FAZ A UNIAO DOS DATAFRAMES
  df_join = pd.concat(df_list)

  return df_join